import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

public class Platno extends JPanel {
	
 	/**
 	 * Bitna slika, v katero rišemo.
 	 */
 	private BufferedImage slika;

 	private int sirina;
	private int visina;
	
	/**
	 * Vlakno, ki računa sliko (če je null, pomeni, da trenutno ne računamo)
	 */
	private Thread vlakno;
	
	/**
	 * Če to nastavimo na true, se mora vlakno čim prej ustaviti
	 */
	private boolean ustavi;

	public Platno(int sirina, int visina) {
		super();
		this.setPreferredSize(new Dimension(sirina, visina));
		this.setBackground(Color.white);
		this.sirina = sirina;
		this.visina = visina;
	}
	
	/**
	 * Sproži risanje slike v vzporednem vlaknu. Če se slika že riše, se trenutno risanje
	 * ustavi in sproži se novo risanje.
	 * 
	 * @throws InterruptedException
	 */
	public void narisi() throws InterruptedException {
		// Ali se trenutno že rise slika?
		if (vlakno != null) {
			// vlaknu povemo, da naj se ustavi
			ustavi = true;
			// počakamo, da se ustavi
			vlakno.join();
		}
		// naredimo novo vlakno: notacija () -> { ... } je ekvivalentna
		// lambda: ... v Pythonu
		vlakno = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					ustavi = false;
					slikar();
					vlakno = null;
				} catch (Exception e) {
				}				
			}
		});		
//		vlakno = new Thread(() -> {
//			try {
//				ustavi = false;
//				this.slikar();
//				vlakno = null;
//			} catch (Exception e) {
//			}				
//		});
		vlakno.start();
	}

	
	/**
	 * Metoda, ki dejansko računa sliko. Tu smo si izmislili neko brezvezno
	 * risanje slike, ki poleg vsega namerno deluje počasi, da se vidi na zaslonu,
	 * kako nastaja slika. Prava aplikacija ne bi namerno delala počasi.
	 * 
	 * @throws InterruptedException
	 */
	private void slikar() throws InterruptedException {
		// ustvarimo novo bitno sliko z dan sirini in visino
		slika = new BufferedImage(sirina, visina, BufferedImage.TYPE_INT_RGB);
		// izracunamo sliko
		for (int y = 0; y < visina; y++) {
			if (ustavi) { return; }
			for (int x = 0; x < sirina; x++) {
				Color c = new Color(
						(x + 3*y) % 100,
						(2*x + y * y) % 255,
						y % 100);
				slika.setRGB(x, y, c.getRGB());
			}
			// počakamo 10 ms (tega v resnem programu ne bi počeli)
			Thread.sleep(10);
			// občasno, recimo vsakih 50 vrstic, narišemo sliko, da se vidi, kako nastaja
			if (y % 50 == 0) { repaint(); }
		}
		// na koncu narišemo sliko
		repaint();
	}
	
	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		if (this.slika != null) {
			// če imamo sliko, jo narišemo
			g.drawImage(slika, 0, 0, getBackground(), null);
		}
		else {
			// če slike ni, napišemo, da je ni
			String napis = "(ni slike)";
			// nastavimo font za izpis
			g.setFont(new Font("Helvetica", Font.PLAIN, 30));
			// dobimo objekt fm, ki zna računati vse v zvezi s fontom
			FontMetrics fm = g.getFontMetrics();
			// objekt fm vprašamo, kako velik bo naš napis, da ga znamo
			// centrirati
			Rectangle2D r = fm.getStringBounds(napis, g);
			// naredimo nais, centrirano
			g.drawString(napis, 
					(int)((getWidth() - r.getWidth())/2),
					(int)((getHeight() - r.getHeight())/2));
		}
	}
	
}
